precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform lowp vec2 location0;
uniform lowp vec2 location1;
uniform lowp vec2 location2;
uniform lowp vec2 location3;
uniform lowp vec2 location4;
uniform lowp vec2 location5;
uniform lowp vec2 location6;
uniform lowp vec2 location7;
uniform lowp vec2 location8;
uniform lowp vec2 location9;
uniform lowp vec2 location10;
uniform lowp vec2 location11;

uniform lowp vec2 location12;
uniform lowp vec2 location13;

uniform lowp float face_ratio;
uniform lowp float eye_ratio;
uniform lowp float shortface_ratio;
uniform lowp float slimNose_ratio;

#define p_faceleft location0
#define p_chin location1
#define p_faceright location2
#define p_nose location3
#define p_eyea location4
#define p_eyeb location5
#define p_left location6
#define p_right location7
#define p_chinleft location8
#define p_chinright location9
#define p_chinleft_u location10
#define p_chinright_u location11

#define p_noseleft location12
#define p_noseright location13

vec2 real_p_faceleft;
vec2 real_p_chin;
vec2 real_p_faceright;
vec2 real_p_nose;
vec2 real_p_eyea;
vec2 real_p_eyeb;
vec2 real_p_left;
vec2 real_p_right;
vec2 real_p_chinleft;
vec2 real_p_chinright;
vec2 real_p_chinleft_u;
vec2 real_p_chinright_u;
vec2 real_p_noseleft;
vec2 real_p_noseright;
vec2 real_cur_pos;


#define x_a 0.72
#define y_a 1.28

#define face_parameter face_ratio
#define eye_parameter eye_ratio
#define glslname_nvshen 0.0

vec4 blendNormal(vec4 c1, vec4 c2)
{
#ifdef USE_NATIVE_MIX
    return mix(c2, c1, c1.a);
#else
    vec4 outputColor;
    outputColor.r = c1.r + c2.r * c2.a * (1.0 - c1.a);
    outputColor.g = c1.g + c2.g * c2.a * (1.0 - c1.a);
    outputColor.b = c1.b + c2.b * c2.a * (1.0 - c1.a);
    outputColor.a = c1.a + c2.a * (1.0 - c1.a);
    return outputColor;
#endif
}

vec2 faceStretch(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius)
{
    vec2 direction = targetPosition - originPosition;
    float lengthA = length(direction);
    float lengthB = min(lengthA, radius);
    direction *= lengthB / lengthA;
    float infect = distance(textureCoord, originPosition)/radius;
    infect = clamp(1.0-infect,0.0,1.0);

    return direction * infect;
}


//点到直线的距离，lineP1、lineP2是过直线的两点
vec2 getPosPointToLine(vec2 pt1,vec2 pt2,vec2 pt3){
    float A = (pt1.y-pt2.y)/(pt1.x- pt2.x);
    float B = (pt1.y-A*pt1.x);
    /// > 0 = ax +b -y;  对应垂线方程为 -x -ay + m = 0;(mm为系数)
    /// > A = a; B = b;
    float m = pt3.x + A*pt3.y;
    
    /// 求两直线交点坐标
    float x = (m-A*B)/(A*A + 1.0);
    float y = A*x+B;
    return vec2(x,y);
}

vec2 narrowNose(vec2 curCoord,vec2 circleCenter,float radius,float intensity)
{
   float disToCenter =distance(curCoord,circleCenter);
   float ratioH = disToCenter/(radius*0.8);   //越往左右ratio越大

    ratioH = step(1.0,ratioH)*clamp(2.0 - ratioH,0.0,1.0) + (1.0 - step(1.0,ratioH))*ratioH;
//   if(ratioH < 1.0){
//       ratioH = ratioH;
//   }else{
//       ratioH = clamp(2.0 - ratioH,0.0,1.0);
//   }

    vec2 posInHLine = getPosPointToLine(real_p_noseleft,real_p_noseright,curCoord);
    float disToHLine = distance(curCoord,posInHLine);
    float ratioV = disToHLine/(radius*0.5);  //越往上下ratio越大
    ratioV = 1.0 - clamp(ratioV,0.0,1.0);

    curCoord = curCoord+(curCoord - circleCenter) *ratioH *ratioV*intensity;     //weight需在0~1之间   weight越大  curCoord变化越小

    return curCoord;
}

//vec2 narrowNose(vec2 curCoord,vec2 circleCenter,float radius,float intensity)
//{
//    float currentDistance = distance(curCoord,circleCenter);
//    //if (currentDistance<=radius)
//    {
//        float weight = currentDistance/radius;
//        weight = 1.0-intensity*(1.0-weight*weight);
//        weight = clamp(weight,0.0001,1.0);
//        curCoord = circleCenter+(curCoord-circleCenter)/weight;
//    }
//    return curCoord;
//}

void main()
{
    
    gl_FragColor = texture2D(inputImageTexture,textureCoordinate);

    vec2 x_y_proportion=vec2(x_a,y_a);

    vec2 newCoord  = textureCoordinate*x_y_proportion;




    if(location3.x>0.03 && location3.y>0.03)
    {

        real_p_faceleft = location0* x_y_proportion;
        real_p_chin = location1* x_y_proportion;
        real_p_faceright = location2* x_y_proportion;
        real_p_nose = location3* x_y_proportion;
        real_p_eyea = location4* x_y_proportion;
        real_p_eyeb = location5* x_y_proportion;
        real_p_left = location6* x_y_proportion;         //左嘴角
        real_p_right = location7* x_y_proportion;        //右嘴角
        real_p_chinleft = location8* x_y_proportion;
        real_p_chinright = location9* x_y_proportion;
        real_p_chinleft_u = location10* x_y_proportion;
        real_p_chinright_u = location11* x_y_proportion;
        real_p_noseleft = location12* x_y_proportion;
        real_p_noseright = location13* x_y_proportion;
        real_cur_pos = textureCoordinate*x_y_proportion;


        float weight = 0.0;
        float face_width = distance(real_p_eyea,real_p_eyeb);

        float eyeRatio = 0.0;
        float faceRatio = 0.0;
        float shortFaceRatio = 0.0;
        faceRatio = face_parameter;
        shortFaceRatio = shortface_ratio;
        // eye1
        float eyeRadius = distance(real_p_left, real_p_right)*0.3;
        float dis_eye1 = distance(newCoord, real_p_eyea);
        if(dis_eye1 <= eyeRadius)
        {
            weight = dis_eye1 / eyeRadius;
            weight = 1.0-0.2*(1.0-weight*weight);
            newCoord = real_p_eyea +(newCoord - real_p_eyea) * weight;
            eyeRatio = eye_parameter;
            faceRatio = 0.0;
            shortFaceRatio = 0.0;
            newCoord = real_cur_pos+(newCoord-real_cur_pos)*eyeRatio;
        }

        // eye2

        float dis_eye2 = distance(newCoord, real_p_eyeb);
        if(dis_eye2 <= eyeRadius)
        {
            weight = dis_eye2 / eyeRadius;
            weight = 1.0-0.2*(1.0-weight*weight);
            newCoord = real_p_eyeb +(newCoord - real_p_eyeb) * weight;
            eyeRatio = eye_parameter;
            faceRatio = 0.0;
            shortFaceRatio = 0.0;
            newCoord = real_cur_pos+(newCoord-real_cur_pos)*eyeRatio;
        }

        vec2 nose = real_p_nose;
        vec2 chin = real_p_chin;


        float radius = face_width;

        float nose_radius =  length(real_p_noseleft - real_p_noseright)*0.5;
        vec2 nose_bottom = real_p_noseleft*0.5 + real_p_noseright*0.5;
        newCoord = narrowNose(newCoord,nose_bottom,nose_radius,slimNose_ratio);

        vec2 leftF = vec2(real_p_faceleft.x, real_p_faceleft.y);
        vec2 targetleftF = real_p_nose + (leftF - real_p_nose) * 0.95;
        vec2 leftFplus = vec2(0.0);
        leftFplus = faceStretch(newCoord, leftF, targetleftF, radius)*faceRatio;
        newCoord = newCoord - leftFplus;

        vec2 rightF = vec2(real_p_faceright.x, real_p_faceright.y);
        vec2 targetrightF = real_p_nose + (rightF - real_p_nose) * 0.95;
        vec2 rightFplus = vec2(0.0);
        rightFplus = faceStretch(newCoord, rightF, targetrightF, radius)*faceRatio;
        newCoord = newCoord - rightFplus;


        radius = face_width*1.5;
        vec2 targetchin = nose + (chin - nose) * 0.92;
        vec2 chinplus = vec2(0.0);
        chinplus = faceStretch(newCoord, chin, targetchin, radius)*(faceRatio+shortFaceRatio);
        newCoord = newCoord - chinplus;


        vec2 chinCenter = real_p_nose + (real_p_chin - real_p_nose) * 0.7;

        radius = face_width*1.2;
        vec2 leftC = vec2(real_p_chinleft.x, real_p_chinleft.y);
        vec2 targetleftC = chinCenter + (leftC - chinCenter) * 0.95;
        vec2 leftCplus = vec2(0.0);
        leftCplus = faceStretch(newCoord, leftC, targetleftC, radius)*faceRatio;
        newCoord = newCoord - leftCplus;

        vec2 rightC = vec2(real_p_chinright.x, real_p_chinright.y);
        vec2 targetrightC = chinCenter + (rightC - chinCenter) * 0.95;
        vec2 rightCplus = vec2(0.0);
        rightCplus = faceStretch(newCoord, rightC, targetrightC, radius)*faceRatio;
        newCoord = newCoord - rightCplus;


        leftC = vec2(real_p_chinleft_u.x, real_p_chinleft_u.y);
        targetleftC = chinCenter + (leftC - chinCenter) * 0.88;
        leftCplus = faceStretch(newCoord, leftC, targetleftC, radius)*faceRatio;
        newCoord = newCoord - leftCplus;

        rightC = vec2(real_p_chinright_u.x, real_p_chinright_u.y);
        targetrightC = chinCenter + (rightC - chinCenter) * 0.88;
        rightCplus = faceStretch(newCoord, rightC, targetrightC, radius)*faceRatio;
        newCoord = newCoord - rightCplus;

        radius=face_width*0.5;
        vec2 srcPoint=real_p_chin;
        vec2 dstPoint=srcPoint+(real_p_chin- real_p_nose)*0.03;//0.02
        vec2 chin_plusBottom = faceStretch(newCoord,srcPoint,dstPoint,radius);
        newCoord = newCoord - chin_plusBottom;

        srcPoint=real_p_chin+(real_p_nose- real_p_chin)*0.5;
        dstPoint=srcPoint+(real_p_chin- real_p_nose)*0.02;
        vec2 chin_plusTop = faceStretch(newCoord,srcPoint,dstPoint,radius);
        newCoord = newCoord - chin_plusTop;

        newCoord = newCoord/x_y_proportion;

        gl_FragColor = texture2D(inputImageTexture, newCoord);



    }

//        vec4 resultC = texture2D(inputImageTexture, textureCoordinate);
//        gl_FragColor = vec4(resultC.rgb*0.5,resultC.a);
}
