
precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;

uniform lowp vec2 adjustSrc;
uniform lowp vec2 adjustDst;
uniform float adjustRadiusRatio;


uniform lowp vec2 location0;
uniform lowp vec2 location1;


#define  p2        location0
#define  p30       location1


vec2 faceStretch(vec2 textureCoord, vec2 originPosition, vec2 targetPosition, float radius, float curve)
{
    vec2 direction = targetPosition - originPosition;
    float lengthA = length(direction);
    float lengthB = min(lengthA, radius);
    direction *= lengthB / lengthA;
    float infect = distance(textureCoord, originPosition)/radius;
    infect = clamp(1.0-infect,0.0,1.0);
    infect = pow(infect, curve);
    
    return direction * infect;
}

void main()
{
    gl_FragColor= texture2D(inputImageTexture, textureCoordinate);
    vec2 adjustDirection=adjustDst-adjustSrc;
    float directionLength=length(adjustDirection);
    if(directionLength>0.0){
        float faceRadius=distance(p2,p30);
        float adjustRadius=faceRadius*0.15*adjustRadiusRatio;
        float adjustLength=faceRadius*0.02;
        vec2 target=adjustSrc+(adjustLength/directionLength)*adjustDirection;
        vec2 offset=faceStretch(textureCoordinate,adjustSrc,target,adjustRadius,1.0);
        vec2 realCoord=textureCoordinate-offset;
        if(realCoord.x>=0.0&&realCoord.x<=1.0&&realCoord.y>=0.0&&realCoord.y<=1.0){
            gl_FragColor= texture2D(inputImageTexture, realCoord);
        }
    }
}
