
precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform float blackRatio;

const lowp vec3 weight = vec3(0.299,0.587,0.114);


vec4 multiply(vec4 base,vec4 overlay){
    float newR=base.r*overlay.r*overlay.r;
    float newG=base.g*overlay.g*overlay.g;
    float newB=base.b*overlay.b*overlay.b;
    
    vec4 result=vec4(newR,newG,newB,base.a);
    return result;
}
vec4 adjustBrightness(vec4 base){
    float newR=pow(base.r,1.06);
    float newG=pow(base.g,1.06);
    float newB=pow(base.b,1.06);
    
    vec4 result=vec4(newR,newG,newB,base.a);
    return result;
}
vec4 adjustSaturation(vec4 src,float ratio){
    vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);
    float mixturePercent = 1.0;
    float luminance = dot(src.rgb, luminanceWeighting);
    vec3 greyScaleColor = vec3(luminance);
    vec4 textureColor3  = vec4(mix(greyScaleColor, src.rgb, ratio), src.w);
    vec4 result=textureColor3;
    return result;
}


float computeSaturation(vec4 src){
    float gray=dot(src.rgb,weight);
    float minSaturation=1.0;
    float maxSaturation=1.19;
    float saturation=1.0;
    
    float minGray=0.25;
    float maxGray=0.35;
    if(gray<minGray){
        float a=(maxSaturation-minSaturation)/(minGray*minGray);
        saturation=a*gray*gray+1.0;
    }
    else if(gray>maxGray){
        float a=(maxSaturation-minSaturation)/((maxGray-1.0)*(maxGray-1.0));
        saturation=a*(gray-1.0)*(gray-1.0)+1.0;
    }
    else{
        saturation=maxSaturation;
    }
    
    return saturation;
}

void main()
{
    vec4 result=texture2D(inputImageTexture,textureCoordinate);
    vec4 curColor=texture2D(inputImageTexture,textureCoordinate);
    
    result=multiply(result,result);
    result=vec4(mix(curColor.rgb, result.rgb, blackRatio),1.0);

    float saturation=computeSaturation(result);
    result=adjustSaturation(curColor,saturation);
    
    result=adjustBrightness(result);
    gl_FragColor = result;
    
}
