
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;

const float resolution = 256.0;

uniform lowp vec2 leftBottom;
uniform lowp vec2 rightTop;

uniform lowp vec2 browLeftBottom;
uniform lowp vec2 browRightTop;

uniform lowp vec2 RleftBottom;
uniform lowp vec2 RrightTop;

uniform lowp vec2 browRLeftBottom;
uniform lowp vec2 browRRightTop;

uniform lowp vec2 location0 ;
uniform lowp vec2 location1 ;
uniform lowp vec2 location2 ;
uniform lowp vec2 location3 ;
uniform lowp vec2 location4 ;
uniform lowp vec2 location5 ;
uniform lowp vec2 location6 ;
uniform lowp vec2 location7 ;
uniform lowp vec2 location8 ;
uniform lowp vec2 location9 ;
uniform lowp vec2 location10;
uniform lowp vec2 location11;
uniform lowp vec2 location12;
uniform lowp vec2 location13;
uniform lowp vec2 location14;

uniform lowp vec2 location15;
uniform lowp vec2 location16;
uniform lowp vec2 location17;
uniform lowp vec2 location18;
uniform lowp vec2 location19;
uniform lowp vec2 location20;
uniform lowp vec2 location21;
uniform lowp vec2 location22;
uniform lowp vec2 location23;
uniform lowp vec2 location24;
uniform lowp vec2 location25;
uniform lowp vec2 location26;
uniform lowp vec2 location27;
uniform lowp vec2 location28;
uniform lowp vec2 location29;

float isInPolygon(vec2 p,vec2 ptPolygon[15],float count){
    float nCross = 0.0;
    for (float i = 0.0; i < count; i++)
    {
        vec2 p1 = ptPolygon[int(i)];
        vec2 p2 = ptPolygon[int(mod((i + 1.0), count))];
        // 求解 y=p.y 与 p1p2 的交点
        if ( p1.y == p2.y )
            // p1p2 与 y=p0.y平行
            continue;
        if ( p.y < min(p1.y, p2.y) ) // 交点在p1p2延长线上
            continue;
        if ( p.y >= max(p1.y, p2.y))// 交点在p1p2延长线上
            continue;
        // 求交点的 X 坐标 --------------------------------------------------------------
        float x = (p.y - p1.y) * (p2.x - p1.x) / (p2.y - p1.y) + p1.x;
        if ( x > p.x )
            nCross++; // 只统计单边交点
    }
    // 单边交点为偶数，点在多边形之外 ---
    return mod(nCross,2.0);
}

float GetGaussianDistribution(float x,float y,float rho) {
    float g = 1.0 / sqrt(2.0 * 3.141592654 * rho * rho);
    return g * exp (-(x * x + y * y) / (2.0 * rho * rho));
}

vec4 GetGaussBlurColor(vec2 uv)
{
    //算出一个像素的空间
    float space = 1.0 / resolution;
    //参考正态分布曲线图，可以知道 3σ 距离以外的点，权重已经微不足道了。
    //反推即可知道当模糊半径为r时，取σ为 r/3 是一个比较合适的取值。
    float blurRadius = 4.0;
    float rho = blurRadius * space / 3.0;
    
    //---权重总和
    float weightTotal = 0.0;
    for(float x = - blurRadius ; x <= blurRadius ; x++)
    {
        for(float y = - blurRadius ; y <= blurRadius ; y++)
        {
            weightTotal += GetGaussianDistribution(x * space, y * space, rho );
        }
    }
    
    vec4 colorTmp = vec4(0.0,0.0,0.0,0.0);
    for(float x = - blurRadius ; x <= blurRadius ; x++)
    {
        for(float y = - blurRadius ; y <= blurRadius ; y++)
        {
            float weight = GetGaussianDistribution( x * space, y * space, rho ) / weightTotal;
            
            vec4 color = texture2D(inputImageTexture,uv + vec2(x * space,y * space));
            color = color * weight;
            colorTmp += color;
        }
    }
    return colorTmp;
}

void main()
{
    vec2 coordinate = vec2(textureCoordinate.x, textureCoordinate.y);
    
    gl_FragColor = texture2D(inputImageTexture, coordinate);
    
    vec2 leftBrow[15];
    leftBrow[0] = location0;
    leftBrow[1] = location1;
    leftBrow[2] = location2;
    leftBrow[3] = location3;
    leftBrow[4] = location4;
    leftBrow[5] = location5;
    leftBrow[6] = location6;
    leftBrow[7] = location7;
    leftBrow[8] = location8;
    leftBrow[9] = location9;
    leftBrow[10] = location10;
    leftBrow[11] = location11;
    leftBrow[12] = location12;
    leftBrow[13] = location13;
    leftBrow[14] = location14;
    
    vec2 rightBrow[15];
    rightBrow[0] = location15;
    rightBrow[1] = location16;
    rightBrow[2] = location17;
    rightBrow[3] = location18;
    rightBrow[4] = location19;
    rightBrow[5] = location20;
    rightBrow[6] = location21;
    rightBrow[7] = location22;
    rightBrow[8] = location23;
    rightBrow[9] = location24;
    rightBrow[10] = location25;
    rightBrow[11] = location26;
    rightBrow[12] = location27;
    rightBrow[13] = location28;
    rightBrow[14] = location29;
    
    float isInLeft = isInPolygon(coordinate,leftBrow,15.0);
    vec4 currentColor = texture2D(inputImageTexture, textureCoordinate);
    vec4 originColor = texture2D(inputImageTexture,textureCoordinate);

    if(isInLeft == 1.0){
        
        currentColor = GetGaussBlurColor(coordinate);
        gl_FragColor = currentColor;

    }else{
        float isInRight = isInPolygon(coordinate,rightBrow,15.0);
        if(isInRight == 1.0){
            
            currentColor = GetGaussBlurColor(coordinate);
            gl_FragColor = currentColor;

        }
    }

    if (coordinate.x >= browLeftBottom.x && coordinate.y >= browLeftBottom.y && coordinate.x <= browRightTop.x && coordinate.y <= browRightTop.y) {
        vec2 test = vec2((coordinate.x - browLeftBottom.x) / (browRightTop.x - browLeftBottom.x), (coordinate.y - browLeftBottom.y) / (browRightTop.y - browLeftBottom.y));
        vec4 browColor = texture2D(inputImageTexture3, test);
        
        float browColorA;
        if (browColor.b > 0.0) {
            browColorA = 1.0;
        }
        else
        {
            browColorA = browColor.a;
        }
        
        gl_FragColor = browColor * browColorA + currentColor * (1.0 - browColor.a);
    }
    
    if (coordinate.x >= browRLeftBottom.x && coordinate.y >= browRLeftBottom.y && coordinate.x <= browRRightTop.x && coordinate.y <= browRRightTop.y) {
        vec2 test = vec2(1.0 - (coordinate.x - browRLeftBottom.x) / (browRRightTop.x - browRLeftBottom.x), (coordinate.y - browRLeftBottom.y) / (browRRightTop.y - browRLeftBottom.y));
        vec4 browColor = texture2D(inputImageTexture3, test);
        
        float browColorA;
        if (browColor.b > 0.0) {
            browColorA = 1.0;
        }
        else
        {
            browColorA = browColor.a;
        }
        
        gl_FragColor = browColor * browColorA + currentColor * (1.0 - browColor.a);
    }
    
}


