precision highp float;
varying highp vec2 textureCoordinate;
varying highp vec2 textureCoordinate2;
//varying highp vec2 textureCoordinate3;

uniform sampler2D inputImageTexture;//src
uniform sampler2D inputImageTexture2;//realtime mouth mask
//uniform sampler2D inputImageTexture3;

uniform float isOrigin;
uniform float ratio;
uniform lowp vec4 lipstickColor;
uniform lowp vec4 avgLipColor;

const lowp vec3 weight = vec3(0.299,0.587,0.114);
const highp vec4 blend=vec4(1.0,0.3333333,0.0,1.0);

vec3 rgb2HSV(vec3 rgb){
    float R=rgb.x;
    float G=rgb.y;
    float B=rgb.z;
    float H;
    float S;
    float V;
    float max=max(max(R,G),B);
    float min=min(min(R,G),B);

    if(max==min&&max==0.0){
        return vec3(360.0,0.0,0.0);
    }
    if(max==min&&max==1.0){
        return vec3(360.0,0.0,1.0);
    }
    if(R ==max){
        H = (G-B)/(max-min);
    }
    if (G ==max){
        H = 2.0 + (B-R)/(max-min);
    }
    if (B == max){
        H = 4.0 + (R-G)/(max-min);
    }
    H = H * 60.0;
    if( H < 0.0){
        H = H + 360.0;
    }
    S=(max-min)/max;
    V=max;
    return vec3(H,S,V);
}

vec3 hsv2RGB(vec3 hsv){
    float h=hsv.x;
    float s=hsv.y;
    float v=hsv.z;
    float r;
    float g;
    float b;
    h =h / 60.0;            // sector 0 to 5
    float i = floor( h );
    float f = h - i;          // factorial part of h
    float p = v * ( 1.0 - s );
    float q = v * ( 1.0 - s * f );
    float t = v * ( 1.0 - s * ( 1.0 - f ) );
    if(int(i)==0){
        r = v;
        g = t;
        b = p;
    }
    else if(int(i)== 1){
        r = q;
        g = v;
        b = p;
    }
    else if(int(i)== 2){
        r = p;
        g = v;
        b = t;
    }
    else if(int(i)==3){
        r = p;
        g = q;
        b = v;
    }
    else if(int(i)==4){
        r = t;
        g = p;
        b = v;
    }
    else{        // case 5:
        r = v;
        g = p;
        b = q;
    }
    return vec3(r,g,b);
}

//柔光混合
vec4 blendSoft(vec4 base, vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;

    if(base.r <= 0.5){
        nr = (2.0 * overlay.r - 1.0) * (sqrt(base.r) - base.r) + base.r;
    }else{
        nr = (2.0 * overlay.r - 1.0)*(base.r - base.r * base.r) + base.r;
    }

    if(base.g <= 0.5){
        ng = (2.0 * overlay.g - 1.0) * (sqrt(base.g) - base.g) + base.g;
    }else{
        ng = (2.0 * overlay.g - 1.0)*(base.g - base.g * base.g) + base.g;
    }

    if(base.b <= 0.5){
        nb = (2.0 * overlay.b - 1.0) * (sqrt(base.b) - base.b) + base.b;
    }else{
        nb = (2.0 * overlay.b - 1.0)*(base.b - base.b * base.b) + base.b;
    }
    vec4 newColor = vec4(nr, ng, nb, base.a);
    return newColor;
}

//叠加
vec4 overlay(vec4 base,vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;

    if(base.r<=0.5){
        nr=2.0*base.r*overlay.r;
    }else{
        nr=1.0-2.0*(1.0-base.r)*(1.0-overlay.r);
    }

    if(base.g<=0.5){
        ng=2.0*base.g*overlay.g;
    }else{
        ng=1.0-2.0*(1.0-base.g)*(1.0-overlay.g);
    }

    if(base.b<=0.5){
        nb=2.0*base.b*overlay.b;
    }else{
        nb=1.0-2.0*(1.0-base.b)*(1.0-overlay.b);
    }

    return vec4(nr,ng,nb,base.a);
}

//滤色
vec4 screen(vec4 base,vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;
    nr=1.0-(1.0-base.r)*(1.0-overlay.r);
    ng=1.0-(1.0-base.g)*(1.0-overlay.g);
    nb=1.0-(1.0-base.b)*(1.0-overlay.b);

    return vec4(nr,ng,nb,base.a);
}

vec4 adjustWhite(vec4 base,float ratio){
    float newR=base.r;
    if(newR>0.0){
        newR=pow(base.r,ratio);
    }

    float newG=base.g;
    if(newG>0.0){
         newG=pow(base.g,ratio);
    }

    float newB=base.b;
    if(newB>0.0){
         newB=pow(base.b,ratio);
    }

    vec4 result=vec4(newR,newG,newB,base.a);
    return result;
}

vec4 obtainNewColor(vec4 src,vec4 dst){
    vec4 blendResult=blendSoft(src,blend);
    vec3 blendResultHSV=rgb2HSV(blendResult.rgb);

    float gray=dot(src.rgb,weight);
    float realS=blendResultHSV.y;
    float realV=blendResultHSV.z;

    vec3 dstHSV=rgb2HSV(dst.rgb);
    float s=realS*dstHSV.y;
    float v=(gray+realV)*0.5*pow(dstHSV.z,1.0-(gray+realV)*0.5);

    vec3 realDstHSV=vec3(dstHSV.x,s,v);
    vec3 realDst=hsv2RGB(realDstHSV);

    return vec4(realDst,src.a);
}

float computeWhiteRatio(vec4 src){
    float thresholdGray=dot(avgLipColor.rgb,weight)*1.15;
    float gray=dot(src.rgb,weight);
    float maxRatio=1.0;
    float minRatio=0.2;
    float whiteRatio=1.0;
    if(gray>thresholdGray){
        float a=(maxRatio-minRatio)/(pow(thresholdGray-1.0,2.0));
        whiteRatio=a*pow(gray-1.0,2.0)+minRatio;
    }
    return whiteRatio;
}
bool isInLip(vec2 coord){
    vec4 color=texture2D(inputImageTexture2,coord);
    return color.g>0.0;
}

bool isValid(float value){
    if(value>=0.0&&value<=1.0){
        return true;
    }
    return false;
}

bool isValidColor(vec3 color){
    return isValid(color.r)&&isValid(color.g)&&isValid(color.b);
}

void main()
{
    vec4 originColor=texture2D(inputImageTexture,textureCoordinate);
    if(isOrigin>0.5){
        gl_FragColor=originColor;
    }else{
            vec4 newColor=obtainNewColor(originColor,lipstickColor);
            vec4 maskColor=texture2D(inputImageTexture2,textureCoordinate2);
            float gradientRatio=maskColor.r;
            gl_FragColor=vec4(mix(originColor.rgb,newColor.rgb,gradientRatio*ratio),originColor.a);

    }
}