
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform float isDrawSrcImage;
uniform float mixCoef;
uniform lowp vec4 wigColor;
uniform lowp vec4 wigColor2;
uniform lowp vec2 gradientLocation;
uniform float isGradient;

const lowp vec3 weight = vec3(0.299,0.587,0.114);

vec3 rgb2HSV(vec3 rgb){
    float R=rgb.x;
    float G=rgb.y;
    float B=rgb.z;
    float H;
    float S;
    float V;
    float max=max(max(R,G),B);
    float min=min(min(R,G),B);
    
    if(max==min&&max<=0.01){
        return vec3(360.0,0.0,0.0);
    }
    if(max==min&&max>=0.99){
        return vec3(360.0,0.0,1.0);
    }
    if(R ==max){
        H = (G-B)/(max-min);
    }
    if (G ==max){
        H = 2.0 + (B-R)/(max-min);
    }
    if (B == max){
        H = 4.0 + (R-G)/(max-min);
    }
    H = H * 60.0;
    if( H < 0.0){
        H = H + 360.0;
    }
    S=(max-min)/max;
    V=max;
    return vec3(H,S,V);
}

vec3 hsv2RGB(vec3 hsv){
    float h=hsv.x;
    float s=hsv.y;
    float v=hsv.z;
    float r;
    float g;
    float b;
    h =h / 60.0;            // sector 0 to 5
    float i = floor( h );
    float f = h - i;          // factorial part of h
    float p = v * ( 1.0 - s );
    float q = v * ( 1.0 - s * f );
    float t = v * ( 1.0 - s * ( 1.0 - f ) );
    if(int(i)==0){
        r = v;
        g = t;
        b = p;
    }
    else if(int(i)== 1){
        r = q;
        g = v;
        b = p;
    }
    else if(int(i)== 2){
        r = p;
        g = v;
        b = t;
    }
    else if(int(i)==3){
        r = p;
        g = q;
        b = v;
    }
    else if(int(i)==4){
        r = t;
        g = p;
        b = v;
    }
    else{        // case 5:
        r = v;
        g = p;
        b = q;
    }
    return vec3(r,g,b);
}

vec4 blendSoft(vec4 base, vec4 overlay){
    //柔光叠加算法
    float nr = base.r;
    if(base.r <= 0.5){
        nr = (2.0 * overlay.r - 1.0)*(base.r - base.r * base.r) + base.r;
    }else{
        nr = (2.0 * overlay.r - 1.0) * (sqrt(base.r) - base.r) + base.r;
    }
    float ng = base.g;
    if(base.g <= 0.5){
        ng = (2.0 * overlay.g - 1.0)*(base.g - base.g * base.g) + base.g;
    }else{
        ng = (2.0 * overlay.g - 1.0) * (sqrt(base.g) - base.g) + base.g;
    }
    float nb = base.b;
    if(base.b <= 0.5){
        nb = (2.0 * overlay.b - 1.0)*(base.b - base.b * base.b) + base.b;
    }else{
        nb = (2.0 * overlay.b - 1.0) * (sqrt(base.b) - base.b) + base.b;
    }
    vec4 newColor = vec4(nr, ng, nb, base.a);
    return newColor;
}
void main()
{
    vec4 originColor = texture2D(inputImageTexture, textureCoordinate);
    gl_FragColor=originColor;

    if(isDrawSrcImage<0.5){
        if(originColor.a>0.0){
            vec4 newColor;
            if(isGradient>0.5){
                float gradientLength=gradientLocation[1]-gradientLocation[0];
                float offset=textureCoordinate.y-gradientLocation[0];
                float ratio=offset/gradientLength;
                if(ratio<0.0){
                ratio=0.0;
                }else  if(ratio>1.0){
                ratio=1.0;
                }
                newColor=mix(wigColor,wigColor2,ratio);
            }
            else{
                newColor=wigColor;
            }
            vec3 hsv=rgb2HSV(newColor.rgb);
            vec3 realHsv=vec3(hsv.x,hsv.y,hsv.z*mixCoef);
            vec4 realMixColor=vec4(hsv2RGB(realHsv),newColor.a);
            float gray=dot(originColor.rgb,weight);
            vec3 grayColor=vec3(gray,gray,gray);
            gl_FragColor=blendSoft(vec4(grayColor*originColor.a,originColor.a),realMixColor);
        }
    }

}
