
precision highp float;
precision highp int;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform lowp vec2 leftEyePoints[4];
uniform lowp vec2 rightEyePoints[4];
uniform lowp vec2 mouthPoints[12];
uniform lowp vec2 insideMouthPoints[8];
uniform lowp vec2 faceContourPoints[19];
uniform lowp vec2 faceForeheadPoints[14];

bool isInLeftEyePolygon(vec2 point,vec2 points[4],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){
            
            continue;
        }
        
        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){
            
            continue;
        }
        
        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){
            
            continue;
        }
        
        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;
        
        // 只统计单边交点
        if (x >= point.x){
            
            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}
bool isInRightEyePolygon(vec2 point,vec2 points[4],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){

            continue;
        }

        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;

        // 只统计单边交点
        if (x >= point.x){

            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}
bool isInMouthPolygon(vec2 point,vec2 points[12],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){

            continue;
        }

        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;

        // 只统计单边交点
        if (x >= point.x){

            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}
bool isInInsideMouthPolygon(vec2 point,vec2 points[8],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){

            continue;
        }

        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;

        // 只统计单边交点
        if (x >= point.x){

            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}
bool isInFacePolygon(vec2 point,vec2 points[19],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){

            continue;
        }

        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;

        // 只统计单边交点
        if (x >= point.x){

            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}

bool isInFaceForeheadPolygon(vec2 point,vec2 points[14],float n) {
    float nCross = 0.0;
    for (float i = 0.0; i < n; i++) {
        vec2 p1 = points[int(i)];
        vec2 p2 = points[int(mod(i+1.0,n))];
        // 求解 y=p.y 与 p1 p2 的交点
        // p1p2 与 y=p0.y平行
        if (p1.y == p2.y){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y < min(p1.y, p2.y)){

            continue;
        }

        // 交点在p1p2延长线上
        if (point.y >= max(p1.y, p2.y)){

            continue;
        }

        // 求交点的 X 坐标
        float x =  (point.y - p1.y) * (p2.x - p1.x)
        / (p2.y - p1.y) + p1.x;

        // 只统计单边交点
        if (x >= point.x){

            nCross++;
        }
    }
    return (int(mod(nCross,2.0)) == 1);
}

bool isLeftEye(vec2 coord){
    return isInLeftEyePolygon(coord,leftEyePoints,4.0);
}
bool isRightEye(vec2 coord){
    return isInRightEyePolygon(coord,rightEyePoints,4.0);
}
bool isMouth(vec2 coord){
    return isInMouthPolygon(coord,mouthPoints,12.0);
}
bool isInsideMouth(vec2 coord){
    return isInInsideMouthPolygon(coord,insideMouthPoints,8.0);
}
bool isInFace(vec2 coord){
    return isInFacePolygon(coord,faceContourPoints,19.0);
}

bool isInFaceForehead(vec2 coord){
    return isInFaceForeheadPolygon(coord,faceForeheadPoints,14.0);
}

void main()
{
    gl_FragColor=vec4(0.0,0.0,0.0,1.0);
    if(isInFace(textureCoordinate)){
        gl_FragColor=vec4(1.0,1.0,1.0,1.0);
        if(isInFaceForehead(textureCoordinate)){
            gl_FragColor=vec4(0.0,1.0,0.0,1.0);
        }
        if(isLeftEye(textureCoordinate)){
            gl_FragColor=vec4(0.0,0.0,0.0,1.0);
        }
        else if(isRightEye(textureCoordinate)){
            gl_FragColor=vec4(0.0,0.0,0.0,1.0);
        }
        else if(isMouth(textureCoordinate)){
            gl_FragColor=vec4(0.0,0.0,1.0,1.0);
            if(isInsideMouth(textureCoordinate)){
                gl_FragColor=vec4(0.0,0.0,0.0,1.0);
            }
        }
    }
}
