
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;


uniform lowp vec2 location0;
uniform lowp vec2 location1;

uniform float singleStepV;
uniform float singleStepH;


#define  p43       location0
#define  p49       location1

bool isExceedFace(vec2 point1,vec2 point2){
    float length=distance(point1,point2);
    float faceLength = distance(p43,p49);
    return length>faceLength;
}

bool isHairColor(vec2 coord){
    vec4 color=texture2D(inputImageTexture2, coord);
    return color.r >0.9 && color.g < 0.1 && color.b < 0.1;
}

bool isForeheadColor(vec2 coord){
    vec4 color=texture2D(inputImageTexture2, coord);
    return color.r == 0.0 && color.g == 0.0 && color.b == 1.0;
}

bool isHair(vec2 coord){
    if(isHairColor(coord)){
        return true;
    }
    
    bool result=true;
    bool continueProcess=true;
    
    vec2 leftTop    =coord;
    vec2 left       =coord;
    vec2 leftBottom =coord;
    vec2 top        =coord;
    vec2 bottom     =coord;
    vec2 rightTop   =coord;
    vec2 right      =coord;
    vec2 rightBottom=coord;
    
    bool reachBound=true;
    while(leftTop.x>=0.0&&leftTop.x<=1.0&&leftTop.y>=0.0&&leftTop.y<=1.0){
        leftTop=leftTop-vec2(singleStepH,singleStepV);
        if(isHairColor(leftTop)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,leftTop)){
            break;
        }
    }
    if(reachBound){
        continueProcess=false;
        result=false;
    }
    
    if(continueProcess){
        reachBound=true;
        while(left.x>=0.0&&left.x<=1.0&&left.y>=0.0&&left.y<=1.0){
            left=left+vec2(-singleStepH,0.0);
            if(isHairColor(left)){
                reachBound=false;
                break;
            }
            if(isExceedFace(coord,left)){
                return false;
            }
        }
        if(reachBound){
            return false;
        }
    }
    
    reachBound=true;
    while(leftBottom.x>=0.0&&leftBottom.x<=1.0&&leftBottom.y>=0.0&&leftBottom.y<=1.0){
        leftBottom=leftBottom+vec2(-singleStepH,singleStepV);
        if(isHairColor(leftBottom)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,leftBottom)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    
    reachBound=true;
    while(top.x>=0.0&&top.x<=1.0&&top.y>=0.0&&top.y<=1.0){
        top=top+vec2(0.0,-singleStepV);
        if(isHairColor(top)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,top)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    
    reachBound=true;
    while(bottom.x>=0.0&&bottom.x<=1.0&&bottom.y>=0.0&&bottom.y<=1.0){
        bottom=bottom+vec2(0.0,singleStepV);
        if(isHairColor(bottom)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,bottom)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    
    
    reachBound=true;
    while(rightTop.x>=0.0&&rightTop.x<=1.0&&rightTop.y>=0.0&&rightTop.y<=1.0){
        rightTop=rightTop+vec2(singleStepH,-singleStepV);
        if(isHairColor(rightTop)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,rightTop)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    
    
    reachBound=true;
    while(right.x>=0.0&&right.x<=1.0&&right.y>=0.0&&right.y<=1.0){
        right=right+vec2(singleStepH,0.0);
        if(isHairColor(right)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,right)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    
    
    reachBound=true;
    while(rightBottom.x>=0.0&&rightBottom.x<=1.0&&rightBottom.y>=0.0&&rightBottom.y<=1.0){
        rightBottom=rightBottom+vec2(singleStepH,singleStepV);
        if(isHairColor(rightBottom)){
            reachBound=false;
            break;
        }
        if(isExceedFace(coord,rightBottom)){
            return false;
        }
    }
    if(reachBound){
        return false;
    }
    return result;
}

void main()
{
    gl_FragColor = vec4(0.0,0.0,0.0,1.0);
    if(isHair(textureCoordinate)){
        gl_FragColor = vec4(1.0,0.0,0.0,1.0);
    }
}
