
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;

uniform lowp vec2 location0;
uniform lowp vec2 location1;
uniform lowp vec2 location2;
uniform lowp vec2 location3;
uniform lowp vec2 location4;
uniform lowp vec2 location5;

//调节瘦脸程度，值越大瘦脸程度越大
//最佳值0.2，值为0.0时无瘦脸效果
uniform float ratio;

uniform float adjustRatio;
uniform vec2 adjustSrc;
uniform vec2 adjustDst;

#define p46       location0
#define p8        location1
#define p1        location2
#define p24       location3
#define p31       location4
#define p16       location5

//点到直线的距离，lineP1、lineP2是过直线的两点
float disPointToLine(vec2 point,vec2 lineP1,vec2 lineP2){
    float A=lineP2.y-lineP1.y;
    float B=lineP1.x-lineP2.x;
    float C=lineP2.x*lineP1.y-lineP1.x*lineP2.y;
    
    float part1=A*point.x+B*point.y+C;
    float part2=pow((pow(A,2.0)+pow(B,2.0)),0.5);
    return abs(part1/part2);
}

void main()
{
    gl_FragColor=texture2D(inputImageTexture, textureCoordinate);
    float faceRadius=distance(p1,p31);
    float radiusH=faceRadius*0.12;
    float radiusV=faceRadius*0.5;
    float disToH=disPointToLine(textureCoordinate,p8,p8+p1-p16);
    float disToV=disPointToLine(textureCoordinate,p8,p46);
    if(disToH<radiusH&&disToV<radiusV){
        vec2 direction=(p46-p8)*ratio*0.1;
        float infect=min((1.0-disToH/radiusH),(1.0-disToV/radiusV));
        vec2 offset=direction*infect;
        vec2 realCoord=textureCoordinate-offset;
        gl_FragColor=texture2D(inputImageTexture, realCoord);
    }
    else{
        disToH=disPointToLine(textureCoordinate,p24,p24+p31-p16);
        disToV=disPointToLine(textureCoordinate,p24,p46);
        if(disToH<radiusH&&disToV<radiusV){
            vec2 direction=(p46-p24)*ratio*0.1;
            float infect=min((1.0-disToH/radiusH),(1.0-disToV/radiusV));
            vec2 offset=direction*infect;
            vec2 realCoord=textureCoordinate-offset;
            gl_FragColor=texture2D(inputImageTexture, realCoord);
        }
    }
    
    float adjustRadius=faceRadius*0.2;
    vec2 adjustDirection=adjustDst-adjustSrc;
    float adjustLength=length(adjustDirection);
    if(adjustLength>0.0&&adjustRatio>0.0){
        vec2 realAdjustDirection=adjustRadius/adjustLength*adjustDirection*adjustRatio;
        float infect = distance(textureCoordinate, adjustSrc)/adjustRadius;
        infect = clamp(1.0-infect,0.0,1.0);
        vec2 offset= realAdjustDirection * infect;
        vec2 realCoord=textureCoordinate-offset;
        gl_FragColor=texture2D(inputImageTexture, realCoord);
    }
}
