
precision highp float;
precision highp int;

varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform float singleStepV;
uniform float singleStepH;
uniform float ratio;
uniform lowp vec4 foundationColor;

const lowp vec3 weight = vec3(0.299,0.587,0.114);
const float MAX_GRADIENT_STEP=6.0;


vec4 getColor(vec2 coord){
    return texture2D(inputImageTexture2,coord);
}

bool isFoundationRegion(vec2 coord){
    vec4 color=getColor(coord);
    return color.r>0.0;
}

float computeRatio(vec4 src){
    float gray=dot(src.rgb,weight);
    float minRatio=0.0;
    float maxRatio=ratio;
    float realRatio=ratio;
    float grayThreshold=0.45;
    
    if(gray<grayThreshold){
        float a=(maxRatio-minRatio)/pow(grayThreshold,2.0);
        realRatio=a*pow(gray,2.0)+minRatio;
    }
    if(realRatio>maxRatio){
        realRatio=maxRatio;
    }
    if(realRatio<minRatio){
        realRatio=minRatio;
    }
    return realRatio;
}

bool isEdge(vec2 coord){
    bool leftTop    =isFoundationRegion(coord+MAX_GRADIENT_STEP*vec2(-singleStepH,-singleStepV));
    bool leftBottom =isFoundationRegion(coord+MAX_GRADIENT_STEP*vec2(-singleStepH, singleStepV));
    if(leftBottom!=leftTop){
        return true;
    }
    bool rightTop   =isFoundationRegion(coord+MAX_GRADIENT_STEP*vec2(singleStepH, -singleStepV));
    if(rightTop!=leftTop){
        return true;
    }
    bool rightBottom=isFoundationRegion(coord+MAX_GRADIENT_STEP*vec2(singleStepH,  singleStepV));
    if(rightBottom!=leftTop){
        return true;
    }
    return false;
}

float computeGradientRatio(vec2 coord){
    vec2 start=coord+MAX_GRADIENT_STEP*vec2(-singleStepH,-singleStepV);
    vec2 curCoord;
    float totalFoundationPointCount=0.0;;
    for(float i=0.0;i<MAX_GRADIENT_STEP;i++){
        for(float j=0.0;j<MAX_GRADIENT_STEP;j++){
            curCoord=start+2.0*vec2(i*singleStepH,j*singleStepV);
            if(isFoundationRegion(curCoord)){
                totalFoundationPointCount++;
            }
        }
    }
    float ratio=totalFoundationPointCount/pow(MAX_GRADIENT_STEP,2.0);
    if(ratio>1.0){
        ratio=1.0;
    }
    else if(ratio<0.0){
        ratio=0.0;
    }
    return ratio;
}

void main()
{
    vec4 curColor=texture2D(inputImageTexture,textureCoordinate);
    gl_FragColor = curColor;
    vec4 foundation=foundationColor;
    if(isFoundationRegion(textureCoordinate)){
        float realRatio=computeRatio(curColor);
        vec4 maskColor=getColor(textureCoordinate);
        float gradientRatio=maskColor.r;
        gl_FragColor=vec4(mix(curColor.rgb,foundation.rgb,realRatio*gradientRatio),curColor.a);
    }
}
