
precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;

uniform float ratio;
uniform lowp vec4 lipstickColor;
uniform lowp vec4 avgLipColor;

const lowp vec3 weight = vec3(0.299,0.587,0.114);
const highp vec4 blend=vec4(1.0,0.3333333,0.0,1.0);

vec3 rgb2HSV(vec3 rgb){
    float R=rgb.x;
    float G=rgb.y;
    float B=rgb.z;
    float H;
    float S;
    float V;
    float max=max(max(R,G),B);
    float min=min(min(R,G),B);
    
    if(max==min&&max==0.0){
        return vec3(360.0,0.0,0.0);
    }
    if(max==min&&max==1.0){
        return vec3(360.0,0.0,1.0);
    }
    if(R ==max){
        H = (G-B)/(max-min);
    }
    if (G ==max){
        H = 2.0 + (B-R)/(max-min);
    }
    if (B == max){
        H = 4.0 + (R-G)/(max-min);
    }
    H = H * 60.0;
    if( H < 0.0){
        H = H + 360.0;
    }
    S=(max-min)/max;
    V=max;
    return vec3(H,S,V);
}

vec3 hsv2RGB(vec3 hsv){
    float h=hsv.x;
    float s=hsv.y;
    float v=hsv.z;
    float r;
    float g;
    float b;
    h =h / 60.0;            // sector 0 to 5
    float i = floor( h );
    float f = h - i;          // factorial part of h
    float p = v * ( 1.0 - s );
    float q = v * ( 1.0 - s * f );
    float t = v * ( 1.0 - s * ( 1.0 - f ) );
    if(int(i)==0){
        r = v;
        g = t;
        b = p;
    }
    else if(int(i)== 1){
        r = q;
        g = v;
        b = p;
    }
    else if(int(i)== 2){
        r = p;
        g = v;
        b = t;
    }
    else if(int(i)==3){
        r = p;
        g = q;
        b = v;
    }
    else if(int(i)==4){
        r = t;
        g = p;
        b = v;
    }
    else{        // case 5:
        r = v;
        g = p;
        b = q;
    }
    return vec3(r,g,b);
}

//柔光混合
vec4 blendSoft(vec4 base, vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;
    
    if(base.r <= 0.5){
        nr = (2.0 * overlay.r - 1.0) * (sqrt(base.r) - base.r) + base.r;
    }else{
        nr = (2.0 * overlay.r - 1.0)*(base.r - base.r * base.r) + base.r;
    }
    
    if(base.g <= 0.5){
        ng = (2.0 * overlay.g - 1.0) * (sqrt(base.g) - base.g) + base.g;
    }else{
        ng = (2.0 * overlay.g - 1.0)*(base.g - base.g * base.g) + base.g;
    }
    
    if(base.b <= 0.5){
        nb = (2.0 * overlay.b - 1.0) * (sqrt(base.b) - base.b) + base.b;
    }else{
        nb = (2.0 * overlay.b - 1.0)*(base.b - base.b * base.b) + base.b;
    }
    vec4 newColor = vec4(nr, ng, nb, base.a);
    return newColor;
}

//叠加
vec4 overlay(vec4 base,vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;
    
    if(base.r<=0.5){
        nr=2.0*base.r*overlay.r;
    }else{
        nr=1.0-2.0*(1.0-base.r)*(1.0-overlay.r);
    }
    
    if(base.g<=0.5){
        ng=2.0*base.g*overlay.g;
    }else{
        ng=1.0-2.0*(1.0-base.g)*(1.0-overlay.g);
    }
    
    if(base.b<=0.5){
        nb=2.0*base.b*overlay.b;
    }else{
        nb=1.0-2.0*(1.0-base.b)*(1.0-overlay.b);
    }
    
    return vec4(nr,ng,nb,base.a);
}



//滤色
vec4 screen(vec4 base,vec4 overlay){
    float nr=base.r;
    float ng=base.g;
    float nb=base.b;
    nr=1.0-(1.0-base.r)*(1.0-overlay.r);
    ng=1.0-(1.0-base.g)*(1.0-overlay.g);
    nb=1.0-(1.0-base.b)*(1.0-overlay.b);
    
    return vec4(nr,ng,nb,base.a);
}

vec4 adjustWhite(vec4 base,float ratio){
    float newR=pow(base.r,ratio);
    if(newR<0.0||newR>1.0){
        newR=base.r;
    }
    float newG=pow(base.g,ratio);
    if(newG<0.0||newG>1.0){
        newG=base.g;
    }
    float newB=pow(base.b,ratio);
    if(newB<0.0||newB>1.0){
        newB=base.b;
    }
    
    vec4 result=vec4(newR,newG,newB,base.a);
    return result;
}

vec4 obtainNewColor(vec4 src,vec4 dst){
    vec4 blendResult=blendSoft(src,blend);
    vec3 blendResultHSV=rgb2HSV(blendResult.rgb);
    
    float gray=dot(src.rgb,weight);
    float realS=blendResultHSV.y;
    float realV=blendResultHSV.z;
    
    vec3 dstHSV=rgb2HSV(dst.rgb);
    float s=realS*dstHSV.y;
    float v=(gray+realV)*0.5*pow(dstHSV.z,1.0-(gray+realV)*0.5);
    
    vec3 realDstHSV=vec3(dstHSV.x,s,v);
    vec3 realDst=hsv2RGB(realDstHSV);
    
    return vec4(realDst,src.a);
}

float computeWhiteRatio(vec4 src){
    float thresholdGray=dot(avgLipColor.rgb,weight)*1.15;
    float gray=dot(src.rgb,weight);
    float maxRatio=1.0;
    float minRatio=0.2;
    float whiteRatio=1.0;
    if(gray>thresholdGray){
        float a=(maxRatio-minRatio)/(pow(thresholdGray-1.0,2.0));
        whiteRatio=a*pow(gray-1.0,2.0)+minRatio;
    }
    return whiteRatio;
}
bool isInLip(vec2 coord){
    vec4 color=texture2D(inputImageTexture2,coord);
    return color.r>0.0;
}

bool isValid(float value){
    if(value>=0.0&&value<=1.0){
        return true;
    }
    return false;
}

bool isValidColor(vec3 color){
    return isValid(color.r)&&isValid(color.g)&&isValid(color.b);
}

//颜色混合模式，  wxx  add
vec4 blendColor(int blendMode, vec4 baseColor, vec4 blendColor) {
    if (blendMode <= 0 || blendMode > 9) { //判空 返回贴纸素材
                                           return blendColor;
    } else if (blendMode == 1) { // 1.正片叠底 Multiply
                                 return  blendColor * baseColor + blendColor * (1.0 - baseColor.a) + baseColor * (1.0 - blendColor.a);
    } else if (blendMode == 2) { // 2.叠加 Overlay
                                 float ra;
                                 if (2.0 * baseColor.r < baseColor.a) {
                                     ra = 2.0 * blendColor.r * baseColor.r + blendColor.r * (1.0 - baseColor.a) + baseColor.r * (1.0 - blendColor.a);
                                 } else {
                                     ra = blendColor.a * baseColor.a - 2.0 * (baseColor.a - baseColor.r) * (blendColor.a - blendColor.r) + blendColor.r * (1.0 - baseColor.a) + baseColor.r * (1.0 - 				blendColor.a);
                                 }
                                 float ga;
                                 if (2.0 * baseColor.g < baseColor.a) {
                                     ga = 2.0 * blendColor.g * baseColor.g + blendColor.g * (1.0 - baseColor.a) + baseColor.g * (1.0 - blendColor.a);
                                 } else {
                                     ga = blendColor.a * baseColor.a - 2.0 * (baseColor.a - baseColor.g) * (blendColor.a - blendColor.g) + blendColor.g * (1.0 - baseColor.a) + baseColor.g * (1.0 - blendColor.a);
                                 }
                                 float ba;
                                 if (2.0 * baseColor.b < baseColor.a) {
                                     ba = 2.0 * blendColor.b * baseColor.b + blendColor.b * (1.0 - baseColor.a) + baseColor.b * (1.0 - blendColor.a);
                                 } else {
                                     ba = blendColor.a * baseColor.a - 2.0 * (baseColor.a - baseColor.b) * (blendColor.a - blendColor.b) + blendColor.b * (1.0 - baseColor.a) + baseColor.b * (1.0 - blendColor.a);
                                 }
                                 return vec4(ra, ga, ba, 1.0);
    } else if (blendMode == 3) { // 3. 线性减淡 Linear Dodage
                                 return  vec4(clamp((baseColor.rgb + baseColor.rgb * (1.0 - blendColor.a)+ blendColor.rgb * (1.0 - baseColor.a) + blendColor.rgb) , vec3(0.0), vec3(1.0)) , blendColor.a);
    } else if (blendMode == 4) { // 4.线性加深 Linear Burn
                                 return vec4(clamp(baseColor.rgb + blendColor.rgb  - vec3(1.0) * blendColor.a, vec3(0.0), vec3(1.0)), baseColor.a);
    } else if (blendMode == 5) { // 5.滤色 Screen
                                 vec4 whiteColor = vec4(1.0);
                                 return whiteColor - ((whiteColor - blendColor) * (whiteColor - baseColor));
    } else if (blendMode == 6) { // 6.柔光 SoftLight
                                 float alphaDivisor = baseColor.a + step(baseColor.a, 0.0);
                                 return baseColor * (blendColor.a * (baseColor / alphaDivisor) + (2.0 * blendColor * (1.0 - (baseColor / alphaDivisor)))) + blendColor * (1.0 - baseColor.a) + baseColor * (1.0 - blendColor.a);
    } else if (blendMode == 7) { // 7.亮光 Vivid Light
                                 return vec4(clamp(baseColor.rgb + baseColor.rgb * (1.0 - blendColor.a)+ blendColor.rgb * (1.0 - baseColor.a) + baseColor.rgb * (2.0*blendColor.rgb - vec3(1.0))/ (2.0*(vec3(1.0)-blendColor.rgb)),vec3(0.),vec3(1.)),blendColor.a);
    } else if (blendMode == 8) { // 8.线性光 Linear Light
                                 return vec4(clamp(baseColor.rgb  + baseColor.rgb * (1.0 - blendColor.a)+ blendColor.rgb * (1.0 - baseColor.a) + 2.0 * blendColor.rgb - vec3(1.0),vec3(0.0), vec3(1.0)),blendColor.a);
    } else if (blendMode == 9) { // 9.差值 Different
                                 return vec4(abs(clamp(blendColor.rgb + baseColor.rgb * (1.0 - blendColor.a)+ blendColor.rgb * (1.0 - baseColor.a) - baseColor.rgb,vec3(-1.),vec3(1.))),blendColor.a);
    }
}

//吴晓新重写：  解决口红有杂色的问题
vec4 obtainNewColor2(vec4 src,vec4 dst){
    //vec4 src2 = blendSoft(src,blend);   //源色和 blend 做柔光混合。感觉是多余，切可能引入颜色值异常，屏蔽掉
    vec3 src2HSV = rgb2HSV(src.rgb);

    //float srcGray=dot(src.rgb,weight);   //也没用

    //float realS=blendResultHSV.y;
    //float realV=blendResultHSV.z;

    vec3 dstHSV = rgb2HSV(dst.rgb);

    float x = dstHSV.x;                 //x 是HSV的h，既 色调；色调直接用目标色的

    //float y = src2HSV.y * dstHSV.y;   //y  是 HSV的 s，既 饱和度；
    float y = src2HSV.y * 0.05 + dstHSV.y * 0.95;  //替换上一行，感觉这样更合理， 原色和目标色在饱和度上，加权求和

    //float z = (srcGray+src2HSV.z) * 0.5 * pow(dstHSV.z,1.0-(srcGray+src2HSV.z) * 0.5);  //z 是HSV的 v 既 亮度
    float z = src2HSV.z;  //替换上一行；亮度直接用原色的就行了么

    vec3 realDst=hsv2RGB(vec3(x,y,z));

    return vec4(realDst,src.a);
}

vec4 obtainNewColor3(vec4 src,vec4 dst){
    //vec4 blendResult=blendSoft(src,blend);
    vec4 blendResult  = blendColor(6,src,blend);    //通过对比测试，判断是这个"柔光混合"环节导致的杂色，把它换成新写的函数，杂色消失，"柔光"是标准化的算法，所以替换后，效果和这轮修改前保持一致

    vec3 blendResultHSV=rgb2HSV(blendResult.rgb);

    float gray=dot(src.rgb,weight);
    float realS=blendResultHSV.y;
    float realV=blendResultHSV.z;

    vec3 dstHSV=rgb2HSV(dst.rgb);
    float s=realS*dstHSV.y;
    float v=(gray+realV)*0.5*pow(dstHSV.z,1.0-(gray+realV)*0.5);

    vec3 realDstHSV=vec3(dstHSV.x,s,v);
    vec3 realDst=hsv2RGB(realDstHSV);

    return vec4(realDst,src.a);
}

void main()
{
    vec4 originColor=texture2D(inputImageTexture,textureCoordinate);
    gl_FragColor=originColor;
    if(isInLip(textureCoordinate)){

        //wxx：在部分手机上阴影区会有亮紫色马赛克，猜测是obtainNewColor函数里的算法导致，直接用下面的blendColor() "叠加"模式替换，应该有类似口红效果，且无杂色问题。
        //vec4 newColor=obtainNewColor(originColor,lipstickColor);
        vec4 newColor = obtainNewColor3(originColor,lipstickColor);  //把口红颜色叠加到原色上去

        vec4 maskColor=texture2D(inputImageTexture2,textureCoordinate);
        float gradientRatio=maskColor.r;

        //下面两行代码是吴晓新为解决口红边缘模糊区遇到脸部黑色阴影时，会出现杂色块，做的尝试。这两行屏蔽后，杂色块消失。
        //float whiteRatio=computeWhiteRatio(texture2D(inputImageTexture3,textureCoordinate));
        //newColor=adjustWhite(newColor,pow(whiteRatio,gradientRatio));

        gl_FragColor=vec4(mix(originColor.rgb,newColor.rgb,ratio*gradientRatio),originColor.a);
    }
    
    
    //    if(isInLip(textureCoordinate)){
    //    vec3 rgb=originColor.rgb;
    //    float R=rgb.x;
    //        float G=rgb.y;
    //        float B=rgb.z;
    //        float max=max(max(R,G),B);
    //        float min=min(min(R,G),B);
    //            float gray=(max+min)*0.5;
    //            vec4 grayColor=vec4(gray,gray,gray,1.0);
    //            vec4 overlayColor=overlay(grayColor,lipstickColor);
    //            overlayColor=mix(grayColor,overlayColor,0.8);
    //            vec4 softColor=blendSoft(overlayColor,overlayColor);
    //            vec4 newColor=mix(overlayColor,softColor,0.5);
    //
    //
    //            vec4 maskColor=texture2D(inputImageTexture2,textureCoordinate);
    //            float gradientRatio=maskColor.r;
    //            gl_FragColor=vec4(mix(originColor.rgb,newColor.rgb,ratio*gradientRatio),originColor.a);
    //        }
    //        vec4 maskColor=texture2D(inputImageTexture2,textureCoordinate);
    //        vec4 grayColor=texture2D(inputImageTexture4,textureCoordinate);
    //
    //        vec4 overlay_color = overlay(grayColor,lipstickColor);
    //        overlay_color = mix(grayColor,overlay_color,0.8);
    //
    //        vec4 soft_color = blendSoft(overlay_color,overlay_color);
    //
    //        vec4 mixColor = vec4(mix(overlay_color.rgb,soft_color.rgb,0.5),1.0);
    //
    //        float gradientRatio = maskColor.r;
    //        gl_FragColor=vec4(mix(originColor.rgb,mixColor.rgb,ratio*gradientRatio),originColor.a);
}
